// bf.c - Simple file encryption implementation using Blowfish cipher.
//        Compile against OpenSSL (http://www.openssl.org)
//
// Usage: bf <input file> <output file> [(E)ncrypt (default) / (D)ecrypt]

#include <stdio.h>
#include <openssl/sha.h>
#include <openssl/blowfish.h>
#include <openssl/evp.h>

int usage(void)
{
	printf("Usage: bf <input file> <output file> [(E)ncrypt (default) / (D)ecrypt]\n");
	return -1;
}

int error(const char *msg)
{
	if (msg != NULL && strlen(msg) > 0)
		fprintf(stderr,"Error: %s\n",msg);
	else
		fprintf(stderr,"Error: Unknonw error!\n",msg);
}

int main(int argc, char **argv)
{
	FILE			*fp_in, *fp_out;
	BF_KEY			key;
	unsigned char	md[SHA_DIGEST_LENGTH], input[8], output[8];
	char			strkey[64];
	int				readin, enc = BF_ENCRYPT;

	if (argc < 3)
		return usage();

	if ((fp_in = fopen(argv[1],"rb")) == NULL)
		return error("Unable to open input file for reading.");

	if ((fp_out = fopen(argv[2],"wb")) == NULL)
		return error("Unable to open output file for writing.");

	if (argc > 3)
	{
		if (tolower(argv[3][0]) == 'd')
			enc = BF_DECRYPT;
	}

	EVP_read_pw_string(strkey,64,"Key: ", ((enc == BF_ENCRYPT) ? 1 : 0));
	SHA1(strkey, strlen(strkey), md);
	BF_set_key(&key,SHA_DIGEST_LENGTH,md);

	memset(input,0,8);
	while ((readin = fread(input,1,8,fp_in)) > 0)
	{
		BF_ecb_encrypt(input,output,&key,enc);
		fwrite(output,1,8,fp_out);
		memset(input,0,8);
	}

	fclose(fp_in);
	fclose(fp_out);

	return 0;
}
